import json
from typing import Any, Dict, List, Final, Optional
from dataclasses import dataclass

import websockets
from websockets.typing import Data

URL_STREAMING: Final = "wss://api.lmnt.com/v1/ai/speech/stream"


@dataclass
class Duration:
    """Duration information for a segment of synthesized speech."""

    text: str
    start: float  # Start time in seconds
    duration: float  # Duration in seconds


@dataclass
class SpeechSessionResponse:
    """Response from a speech session connection."""

    audio: bytes
    durations: Optional[List[Duration]] = None
    warning: Optional[str] = None
    buffer_empty: Optional[bool] = None


class UnexpectedMessageError(Exception):
    """Exception raised when an unexpected message is received from the server."""

    def __init__(self, message: str):
        self.message = message
        super().__init__(f"Unexpected message received from server: {message}")


class SpeechSession:
    """Async websocket connection for LMNT speech session."""

    def __init__(
        self,
        api_key: str,
        voice: str,
        format: Optional[str] = None,
        language: Optional[str] = None,
        sample_rate: Optional[int] = None,
        return_extras: Optional[bool] = None,
    ):
        self.api_key = api_key
        self.voice = voice
        self.format = format
        self.language = language
        self.sample_rate = sample_rate
        self.return_extras = return_extras
        self.websocket: Optional[Any] = None

    async def connect(self) -> None:
        """Establish the websocket connection."""
        self.websocket = await websockets.connect(URL_STREAMING)
        init_msg = {
            "X-API-Key": self.api_key,
            "voice": self.voice,
        }
        if self.format:
            init_msg["format"] = self.format
        if self.language:
            init_msg["language"] = self.language
        if self.sample_rate:
            init_msg["sample_rate"] = str(self.sample_rate)
        if self.return_extras:
            init_msg["return_extras"] = str(self.return_extras)
        if self.websocket is not None:
            await self.websocket.send(json.dumps(init_msg))

    async def append_text(self, text: str) -> None:
        """Append text to be synthesized."""
        await self._send_message({"text": text})

    async def flush(self) -> None:
        """Flush the current text buffer."""
        await self._send_message({"flush": True})

    async def finish(self) -> None:
        """Mark the session as finished."""
        await self._send_message({"eof": True})

    async def close(self) -> None:
        """Close the websocket connection."""
        if self.websocket is not None:
            await self.websocket.close()
            self.websocket = None

    async def _send_message(self, message: Dict[str, Any]) -> None:
        """Send a message through the websocket."""
        if self.websocket is not None:
            await self.websocket.send(json.dumps(message))

    def __aiter__(self) -> "SpeechSession":
        """Return the async iterator."""
        return self

    async def __anext__(self) -> SpeechSessionResponse:
        """Get the next speech session response."""
        if not self.websocket:
            raise StopAsyncIteration
        try:
            if self.return_extras:
                extras_message = await self.websocket.recv()
                if not isinstance(extras_message, str):
                    raise UnexpectedMessageError("Expected string for extras message")
                extras = self._parse_and_check_error(extras_message)
                audio_msg = await self.websocket.recv()
                audio_data = self._process_audio_data(audio_msg)
                durations = None
                if extras.get("durations"):
                    durations = [Duration(**d) for d in extras["durations"]]
                return SpeechSessionResponse(
                    audio=audio_data,
                    durations=durations,
                    warning=extras.get("warning"),
                    buffer_empty=extras.get("buffer_empty"),
                )
            else:
                audio_msg = await self.websocket.recv()
                audio_data = self._process_audio_data(audio_msg)
                return SpeechSessionResponse(audio=audio_data)
        except websockets.exceptions.ConnectionClosed as err:
            raise StopAsyncIteration from err

    def _process_audio_data(self, audio_msg: Data) -> bytes:
        """Process the audio data. Handles binary audio data and JSON error messages."""
        if isinstance(audio_msg, bytes):
            return audio_msg
        else:
            self._parse_and_check_error(str(audio_msg))
        raise UnexpectedMessageError(str(audio_msg))

    def _parse_and_check_error(self, message: str) -> Dict[str, Any]:
        """JSON parse a message and check for errors."""
        try:
            msg_json: Dict[str, Any] = json.loads(message)
        except json.JSONDecodeError as err:
            raise ValueError(f"Invalid JSON received from server: {message}") from err
        if "error" in msg_json:
            raise ValueError(msg_json["error"])
        return msg_json



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .speech import (
    SpeechResource,
    AsyncSpeechResource,
    SpeechResourceWithRawResponse,
    AsyncSpeechResourceWithRawResponse,
    SpeechResourceWithStreamingResponse,
    AsyncSpeechResourceWithStreamingResponse,
)
from .voices import (
    VoicesResource,
    AsyncVoicesResource,
    VoicesResourceWithRawResponse,
    AsyncVoicesResourceWithRawResponse,
    VoicesResourceWithStreamingResponse,
    AsyncVoicesResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from .sessions import AsyncSessionsResource

__all__ = [
    "SpeechResource",
    "AsyncSpeechResource",
    "SpeechResourceWithRawResponse",
    "AsyncSpeechResourceWithRawResponse",
    "SpeechResourceWithStreamingResponse",
    "AsyncSpeechResourceWithStreamingResponse",
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
    "VoicesResource",
    "AsyncVoicesResource",
    "VoicesResourceWithRawResponse",
    "AsyncVoicesResourceWithRawResponse",
    "VoicesResourceWithStreamingResponse",
    "AsyncVoicesResourceWithStreamingResponse",
    "AsyncSessionsResource",
]



from typing import Literal, Optional

from .._resource import AsyncAPIResource
from ..lib.websocket_streaming import SpeechSession

__all__ = ["AsyncSessionsResource", "SpeechSession"]


class AsyncSessionsResource(AsyncAPIResource):
    """Methods for the Sessions resource."""

    async def create(
        self,
        *,
        voice: str,
        format: Optional[Literal["mp3", "raw", "ulaw", "webm"]] = None,
        language: Optional[str] = None,
        sample_rate: Optional[Literal[8000, 16000, 24000]] = None,
        return_extras: Optional[bool] = None,
    ) -> SpeechSession:
        """Create a new websocket connection for full-duplex streaming speech synthesis.
        The websocket connection is cleanly closed when the user calls finish().

        Args:
            voice: The voice id of the voice to use for synthesis
            format: The file format of the synthesized audio output ('mp3', 'raw', 'ulaw', 'webm')
            language: The desired language of the synthesized speech. Two letter ISO 639-1 code.
            sample_rate: The desired output sample rate in Hz (8000, 16000, or 24000)
            return_extras: If True, response will contain a durations object.

        Returns:
            SpeechSession: A new streaming speech session
        """
        session = SpeechSession(
            self._client.api_key,
            voice=voice,
            format=format,
            language=language,
            sample_rate=sample_rate,
            return_extras=return_extras,
        )
        await session.connect()
        return session



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
import logging
from typing import TYPE_CHECKING, Iterator, AsyncIterator

import httpx
import pytest
from pytest_asyncio import is_async_test

from lmnt import Lmnt, AsyncLmnt, DefaultAioHttpClient
from lmnt._utils import is_dict

if TYPE_CHECKING:
    from _pytest.fixtures import FixtureRequest  # pyright: ignore[reportPrivateImportUsage]

pytest.register_assert_rewrite("tests.utils")

logging.getLogger("lmnt").setLevel(logging.DEBUG)


# automatically add `pytest.mark.asyncio()` to all of our async tests
# so we don't have to add that boilerplate everywhere
def pytest_collection_modifyitems(items: list[pytest.Function]) -> None:
    pytest_asyncio_tests = (item for item in items if is_async_test(item))
    session_scope_marker = pytest.mark.asyncio(loop_scope="session")
    for async_test in pytest_asyncio_tests:
        async_test.add_marker(session_scope_marker, append=False)

    # We skip tests that use both the aiohttp client and respx_mock as respx_mock
    # doesn't support custom transports.
    for item in items:
        if "async_client" not in item.fixturenames or "respx_mock" not in item.fixturenames:
            continue

        if not hasattr(item, "callspec"):
            continue

        async_client_param = item.callspec.params.get("async_client")
        if is_dict(async_client_param) and async_client_param.get("http_client") == "aiohttp":
            item.add_marker(pytest.mark.skip(reason="aiohttp client is not compatible with respx_mock"))


base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")

api_key = "My API Key"


@pytest.fixture(scope="session")
def client(request: FixtureRequest) -> Iterator[Lmnt]:
    strict = getattr(request, "param", True)
    if not isinstance(strict, bool):
        raise TypeError(f"Unexpected fixture parameter type {type(strict)}, expected {bool}")

    with Lmnt(base_url=base_url, api_key=api_key, _strict_response_validation=strict) as client:
        yield client


@pytest.fixture(scope="session")
async def async_client(request: FixtureRequest) -> AsyncIterator[AsyncLmnt]:
    param = getattr(request, "param", True)

    # defaults
    strict = True
    http_client: None | httpx.AsyncClient = None

    if isinstance(param, bool):
        strict = param
    elif is_dict(param):
        strict = param.get("strict", True)
        assert isinstance(strict, bool)

        http_client_type = param.get("http_client", "httpx")
        if http_client_type == "aiohttp":
            http_client = DefaultAioHttpClient()
    else:
        raise TypeError(f"Unexpected fixture parameter type {type(param)}, expected bool or dict")

    async with AsyncLmnt(
        base_url=base_url, api_key=api_key, _strict_response_validation=strict, http_client=http_client
    ) as client:
        yield client



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["VoiceUpdateParams"]


class VoiceUpdateParams(TypedDict, total=False):
    description: str
    """A description of this voice."""

    gender: str
    """A tag describing the gender of this voice, e.g. `male`, `female`, `nonbinary`."""

    name: str
    """The display name for this voice."""

    starred: bool
    """If `true`, adds this voice to your starred list."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import FileTypes

__all__ = ["SpeechConvertParams"]


class SpeechConvertParams(TypedDict, total=False):
    audio: Required[FileTypes]
    """The audio file to be converted into a new voice.

    Specify source language using the `language` parameter. Acceptable formats:
    `wav`, `mp3`. Max file size: 1 MB.
    """

    voice: Required[str]
    """The voice id to convert the speech into.

    Voice ids can be retrieved by calls to `List voices` or `Voice info`.
    """

    format: Literal["aac", "mp3", "raw", "ulaw", "wav", "webm"]
    """The desired output format of the audio.

    If you are using a streaming endpoint, you'll generate audio faster by selecting
    a streamable format since chunks are encoded and returned as they're generated.
    For non-streamable formats, the entire audio will be synthesized before
    encoding.

    Streamable formats:

    - `mp3`: 96kbps MP3 audio.
    - `raw`: 32-bit floating point raw audio.
    - `ulaw`: 8-bit G711 µ-law audio with a WAV header.
    - `webm`: WebM format with Opus audio codec.

    Non-streamable formats:

    - `aac`: AAC audio codec.
    - `wav`: 16-bit PCM audio in WAV container.
    """

    language: Literal[
        "auto",
        "ar",
        "de",
        "en",
        "es",
        "fr",
        "hi",
        "id",
        "it",
        "ja",
        "ko",
        "nl",
        "pl",
        "pt",
        "ru",
        "sv",
        "th",
        "tr",
        "uk",
        "ur",
        "vi",
        "zh",
    ]
    """The language of the source audio. Two letter ISO 639-1 code."""

    sample_rate: Literal[8000, 16000, 24000]
    """The desired output sample rate in Hz.

    Defaults to `24000` for all formats except `mulaw` which defaults to `8000`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["AccountRetrieveResponse", "Plan", "Usage"]


class Plan(BaseModel):
    character_limit: int
    """The number of characters you are allowed to synthesize in this billing period."""

    commercial_use_allowed: bool

    professional_voice_limit: Optional[int] = None
    """The number of professional voices you are allowed to create."""

    type: str
    """The type of plan you are subscribed to."""

    instant_voice_limit: Optional[int] = None
    """The number of instant voices you are allowed to create."""


class Usage(BaseModel):
    characters: int
    """The number of characters you have synthesized in this billing period."""

    professional_voices: int
    """The number of professional voices you have created."""

    instant_voices: Optional[int] = None
    """The number of instant voices you have created."""


class AccountRetrieveResponse(BaseModel):
    plan: Plan

    usage: Usage



import asyncio

from lmnt import AsyncLmnt


async def main() -> None:
    client = AsyncLmnt()

    # Use async streaming response to get chunks as they arrive
    async with client.speech.with_streaming_response.generate(
        text="""Okay that's the 4-piece Original Combo with coleslaw and iced tea, and the two-pack of Nashville Hot sliders. Before I get you the total, uh, have you tried our new Peach Cobbler? It's warm, got a buttery crust, folks are loving it - only $2.99 extra.""",
        voice="leah",
    ) as response:
        print(f"Response headers: {response.headers}")

        # Collect all chunks and print when each arrives
        all_content = b""
        chunk_count = 0

        async for chunk in response.iter_bytes():
            chunk_count += 1
            print(f"Chunk {chunk_count} received: {len(chunk)} bytes")
            all_content += chunk

        print(f"Total chunks received: {chunk_count}")
        print(f"Total content size: {len(all_content)} bytes")

    # Write the complete audio to file
    with open("output_async.wav", "wb") as f:
        f.write(all_content)

    print("Audio saved to output_async.wav")


if __name__ == "__main__":
    asyncio.run(main())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["VoiceListParams"]


class VoiceListParams(TypedDict, total=False):
    owner: str
    """Which owner's voices to return. Choose from `system`, `me`, or `all`."""

    starred: str
    """If true, only returns voices that you have starred."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SpeechGenerateDetailedParams"]


class SpeechGenerateDetailedParams(TypedDict, total=False):
    text: Required[str]
    """The text to synthesize; max 5000 characters per request (including spaces)."""

    voice: Required[str]
    """
    The voice id of the voice to use; voice ids can be retrieved by calls to
    `List voices` or `Voice info`.
    """

    format: Literal["aac", "mp3", "raw", "ulaw", "wav", "webm"]
    """The desired output format of the audio.

    If you are using a streaming endpoint, you'll generate audio faster by selecting
    a streamable format since chunks are encoded and returned as they're generated.
    For non-streamable formats, the entire audio will be synthesized before
    encoding.

    Streamable formats:

    - `mp3`: 96kbps MP3 audio.
    - `raw`: 32-bit floating point raw audio.
    - `ulaw`: 8-bit G711 µ-law audio with a WAV header.
    - `webm`: WebM format with Opus audio codec.

    Non-streamable formats:

    - `aac`: AAC audio codec.
    - `wav`: 16-bit PCM audio in WAV container.
    """

    language: Literal[
        "auto",
        "ar",
        "de",
        "en",
        "es",
        "fr",
        "hi",
        "id",
        "it",
        "ja",
        "ko",
        "nl",
        "pl",
        "pt",
        "ru",
        "sv",
        "th",
        "tr",
        "uk",
        "ur",
        "vi",
        "zh",
    ]
    """The desired language.

    Two letter ISO 639-1 code. Defaults to auto language detection.
    """

    model: Literal["blizzard"]
    """The model to use for synthesis.

    Learn more about models [here](https://docs.lmnt.com/guides/models).
    """

    return_durations: bool
    """If set as `true`, response will contain a durations object."""

    sample_rate: Literal[8000, 16000, 24000]
    """The desired output sample rate in Hz.

    Defaults to `24000` for all formats except `mulaw` which defaults to `8000`.
    """

    seed: int
    """Seed used to specify a different take; defaults to random"""

    temperature: float
    """Influences how expressive and emotionally varied the speech becomes.

    Lower values (like 0.3) create more neutral, consistent speaking styles. Higher
    values (like 1.0) allow for more dynamic emotional range and speaking styles.
    """

    top_p: float
    """Controls the stability of the generated speech.

    A lower value (like 0.3) produces more consistent, reliable speech. A higher
    value (like 0.9) gives more flexibility in how words are spoken, but might
    occasionally produce unusual intonations or speech patterns.
    """



import asyncio

from lmnt import AsyncLmnt
from lmnt.resources.sessions import SpeechSession


async def main() -> None:
    # Gets API Key from environment variable LMNT_API_KEY
    lmnt = AsyncLmnt()

    # Construct the streaming connection with our desired voice
    session = await lmnt.speech.sessions.create(voice="leah", return_extras=True)
    write_task = asyncio.create_task(write_messages(session))
    read_task = asyncio.create_task(read_messages(session))

    await asyncio.gather(write_task, read_task)


async def write_messages(session: SpeechSession) -> None:
    # Simulate a message stream w/ a 1 second delay between messages
    for i in range(5):
        await session.append_text("Hello, world!")
        print(f" ** Sent to LMNT -- Message {i} ** ")
        await asyncio.sleep(1)

    # After finish is called, the server will flush its buffer and close the
    # connection once all the audio has been synthesized
    await session.finish()


async def read_messages(session: SpeechSession) -> None:
    with open("stream-output.mp3", "wb") as audio_file:
        async for message in session:
            audio_bytes = len(message.audio)
            print(f" ** Received from LMNT -- {audio_bytes} bytes ** ")
            print(f" ** Durations: {message.durations} ** ")
            audio_file.write(message.audio)


if __name__ == "__main__":
    asyncio.run(main())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from lmnt import Lmnt, AsyncLmnt
from lmnt.types import (
    Voice,
    VoiceListResponse,
    VoiceDeleteResponse,
    VoiceUpdateResponse,
)
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVoices:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism bug detailed here: https://github.com/stoplightio/prism/pull/2654")
    @parametrize
    def test_method_create(self, client: Lmnt) -> None:
        voice = client.voices.create(
            enhance=False,
            files=[b"raw file contents"],
            name="new-voice",
        )
        assert_matches_type(Voice, voice, path=["response"])

    @pytest.mark.skip(reason="Prism bug detailed here: https://github.com/stoplightio/prism/pull/2654")
    @parametrize
    def test_method_create_with_all_params(self, client: Lmnt) -> None:
        voice = client.voices.create(
            enhance=False,
            files=[b"raw file contents"],
            name="new-voice",
            description="description",
            gender="gender",
        )
        assert_matches_type(Voice, voice, path=["response"])

    @pytest.mark.skip(reason="Prism bug detailed here: https://github.com/stoplightio/prism/pull/2654")
    @parametrize
    def test_raw_response_create(self, client: Lmnt) -> None:
        response = client.voices.with_raw_response.create(
            enhance=False,
            files=[b"raw file contents"],
            name="new-voice",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(Voice, voice, path=["response"])

    @pytest.mark.skip(reason="Prism bug detailed here: https://github.com/stoplightio/prism/pull/2654")
    @parametrize
    def test_streaming_response_create(self, client: Lmnt) -> None:
        with client.voices.with_streaming_response.create(
            enhance=False,
            files=[b"raw file contents"],
            name="new-voice",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(Voice, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Lmnt) -> None:
        voice = client.voices.retrieve(
            "id",
        )
        assert_matches_type(Voice, voice, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Lmnt) -> None:
        response = client.voices.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(Voice, voice, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Lmnt) -> None:
        with client.voices.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(Voice, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Lmnt) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.voices.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Lmnt) -> None:
        voice = client.voices.update(
            id="123",
        )
        assert_matches_type(VoiceUpdateResponse, voice, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Lmnt) -> None:
        voice = client.voices.update(
            id="123",
            description="description",
            gender="gender",
            name="name",
            starred=True,
        )
        assert_matches_type(VoiceUpdateResponse, voice, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Lmnt) -> None:
        response = client.voices.with_raw_response.update(
            id="123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(VoiceUpdateResponse, voice, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Lmnt) -> None:
        with client.voices.with_streaming_response.update(
            id="123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(VoiceUpdateResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Lmnt) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.voices.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: Lmnt) -> None:
        voice = client.voices.list()
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Lmnt) -> None:
        voice = client.voices.list(
            owner="owner",
            starred="starred",
        )
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Lmnt) -> None:
        response = client.voices.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Lmnt) -> None:
        with client.voices.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(VoiceListResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Lmnt) -> None:
        voice = client.voices.delete(
            "id",
        )
        assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Lmnt) -> None:
        response = client.voices.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Lmnt) -> None:
        with client.voices.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Lmnt) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.voices.with_raw_response.delete(
                "",
            )


class TestAsyncVoices:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism bug detailed here: https://github.com/stoplightio/prism/pull/2654")
    @parametrize
    async def test_method_create(self, async_client: AsyncLmnt) -> None:
        voice = await async_client.voices.create(
            enhance=False,
            files=[b"raw file contents"],
            name="new-voice",
        )
        assert_matches_type(Voice, voice, path=["response"])

    @pytest.mark.skip(reason="Prism bug detailed here: https://github.com/stoplightio/prism/pull/2654")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLmnt) -> None:
        voice = await async_client.voices.create(
            enhance=False,
            files=[b"raw file contents"],
            name="new-voice",
            description="description",
            gender="gender",
        )
        assert_matches_type(Voice, voice, path=["response"])

    @pytest.mark.skip(reason="Prism bug detailed here: https://github.com/stoplightio/prism/pull/2654")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLmnt) -> None:
        response = await async_client.voices.with_raw_response.create(
            enhance=False,
            files=[b"raw file contents"],
            name="new-voice",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(Voice, voice, path=["response"])

    @pytest.mark.skip(reason="Prism bug detailed here: https://github.com/stoplightio/prism/pull/2654")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLmnt) -> None:
        async with async_client.voices.with_streaming_response.create(
            enhance=False,
            files=[b"raw file contents"],
            name="new-voice",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(Voice, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncLmnt) -> None:
        voice = await async_client.voices.retrieve(
            "id",
        )
        assert_matches_type(Voice, voice, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncLmnt) -> None:
        response = await async_client.voices.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(Voice, voice, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncLmnt) -> None:
        async with async_client.voices.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(Voice, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncLmnt) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.voices.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncLmnt) -> None:
        voice = await async_client.voices.update(
            id="123",
        )
        assert_matches_type(VoiceUpdateResponse, voice, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncLmnt) -> None:
        voice = await async_client.voices.update(
            id="123",
            description="description",
            gender="gender",
            name="name",
            starred=True,
        )
        assert_matches_type(VoiceUpdateResponse, voice, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncLmnt) -> None:
        response = await async_client.voices.with_raw_response.update(
            id="123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(VoiceUpdateResponse, voice, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncLmnt) -> None:
        async with async_client.voices.with_streaming_response.update(
            id="123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(VoiceUpdateResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncLmnt) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.voices.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncLmnt) -> None:
        voice = await async_client.voices.list()
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLmnt) -> None:
        voice = await async_client.voices.list(
            owner="owner",
            starred="starred",
        )
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLmnt) -> None:
        response = await async_client.voices.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLmnt) -> None:
        async with async_client.voices.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(VoiceListResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncLmnt) -> None:
        voice = await async_client.voices.delete(
            "id",
        )
        assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncLmnt) -> None:
        response = await async_client.voices.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncLmnt) -> None:
        async with async_client.voices.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncLmnt) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.voices.with_raw_response.delete(
                "",
            )



from lmnt import Lmnt

client = Lmnt()

# Use streaming response to get chunks as they arrive
with client.speech.with_streaming_response.generate(
    text="""Okay that's the 4-piece Original Combo with coleslaw and iced tea, and the two-pack of Nashville Hot sliders. Before I get you the total, uh, have you tried our new Peach Cobbler? It's warm, got a buttery crust, folks are loving it - only $2.99 extra.""",
    voice="leah",
) as response:
    # Collect all chunks and print when each arrives
    all_content = b""
    chunk_count = 0

    for chunk in response.iter_bytes():
        chunk_count += 1
        print(f"Chunk {chunk_count} received: {len(chunk)} bytes")
        all_content += chunk

    print(f"Total chunks received: {chunk_count}")
    print(f"Total content size: {len(all_content)} bytes")

# Write the complete audio to file
with open("output.wav", "wb") as f:
    f.write(all_content)

print("Audio saved to output.wav")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .voice import Voice as Voice
from .voice_list_params import VoiceListParams as VoiceListParams
from .voice_create_params import VoiceCreateParams as VoiceCreateParams
from .voice_list_response import VoiceListResponse as VoiceListResponse
from .voice_update_params import VoiceUpdateParams as VoiceUpdateParams
from .speech_convert_params import SpeechConvertParams as SpeechConvertParams
from .voice_delete_response import VoiceDeleteResponse as VoiceDeleteResponse
from .voice_update_response import VoiceUpdateResponse as VoiceUpdateResponse
from .speech_generate_params import SpeechGenerateParams as SpeechGenerateParams
from .account_retrieve_response import AccountRetrieveResponse as AccountRetrieveResponse
from .speech_generate_detailed_params import SpeechGenerateDetailedParams as SpeechGenerateDetailedParams
from .speech_generate_detailed_response import SpeechGenerateDetailedResponse as SpeechGenerateDetailedResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Voice"]


class Voice(BaseModel):
    id: str
    """The unique identifier of this voice."""

    name: str
    """The display name of this voice."""

    owner: Literal["system", "me", "other"]
    """The owner of this voice."""

    state: str
    """The state of this voice in the training pipeline (e.g., `ready`, `training`)."""

    description: Optional[str] = None
    """A text description of this voice."""

    gender: Optional[str] = None
    """A tag describing the gender of this voice, e.g. `male`, `female`, `nonbinary`."""

    preview_url: Optional[str] = None
    """A URL that returns a preview speech sample of this voice.

    The file can be played directly in a browser or audio player.
    """

    starred: Optional[bool] = None
    """Whether this voice has been starred by you or not."""

    type: Optional[Literal["instant", "professional"]] = None
    """The method by which this voice was created: `instant` or `professional`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["SpeechGenerateDetailedResponse", "Duration"]


class Duration(BaseModel):
    duration: float
    """The spoken duration of each synthesized input element, in seconds."""

    start: float
    """The start time of each synthsized input element, in seconds."""

    text: str
    """The synthesized input elements; beginning and ending with a short silence."""


class SpeechGenerateDetailedResponse(BaseModel):
    audio: str
    """
    The base64-encoded audio file; the format is determined by the `format`
    parameter.
    """

    seed: int
    """
    The seed used to generate this speech; can be used to replicate this output take
    (assuming the same text is resynthsized with this seed number,
    [see here](http://docs.lmnt.com/speech/seed) for more details).
    """

    durations: Optional[List[Duration]] = None
    """
    A JSON object outlining the spoken duration of each synthesized input element
    (words and non-words like spaces, punctuation, etc.). See an
    [example of this object](https://imgur.com/Uw6qNzY.png) for the input string
    "Hello world!"
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .voice import Voice
from .._models import BaseModel

__all__ = ["VoiceUpdateResponse"]


class VoiceUpdateResponse(BaseModel):
    voice: Voice
    """Voice details"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast
from typing_extensions import Literal

import httpx

from ..types import speech_convert_params, speech_generate_params, speech_generate_detailed_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .sessions import AsyncSessionsResource
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.speech_generate_detailed_response import SpeechGenerateDetailedResponse

__all__ = ["SpeechResource", "AsyncSpeechResource"]


class SpeechResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lmnt-com/lmnt-python#accessing-raw-response-data-eg-headers
        """
        return SpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lmnt-com/lmnt-python#with_streaming_response
        """
        return SpeechResourceWithStreamingResponse(self)

    def convert(
        self,
        *,
        audio: FileTypes,
        voice: str,
        format: Literal["aac", "mp3", "raw", "ulaw", "wav", "webm"] | NotGiven = NOT_GIVEN,
        language: Literal[
            "auto",
            "ar",
            "de",
            "en",
            "es",
            "fr",
            "hi",
            "id",
            "it",
            "ja",
            "ko",
            "nl",
            "pl",
            "pt",
            "ru",
            "sv",
            "th",
            "tr",
            "uk",
            "ur",
            "vi",
            "zh",
        ]
        | NotGiven = NOT_GIVEN,
        sample_rate: Literal[8000, 16000, 24000] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BinaryAPIResponse:
        """
        Converts speech from one voice to another.

        Args:
          audio: The audio file to be converted into a new voice. Specify source language using
              the `language` parameter. Acceptable formats: `wav`, `mp3`. Max file size: 1 MB.

          voice: The voice id to convert the speech into. Voice ids can be retrieved by calls to
              `List voices` or `Voice info`.

          format: The desired output format of the audio. If you are using a streaming endpoint,
              you'll generate audio faster by selecting a streamable format since chunks are
              encoded and returned as they're generated. For non-streamable formats, the
              entire audio will be synthesized before encoding.

              Streamable formats:

              - `mp3`: 96kbps MP3 audio.
              - `raw`: 32-bit floating point raw audio.
              - `ulaw`: 8-bit G711 µ-law audio with a WAV header.
              - `webm`: WebM format with Opus audio codec.

              Non-streamable formats:

              - `aac`: AAC audio codec.
              - `wav`: 16-bit PCM audio in WAV container.

          language: The language of the source audio. Two letter ISO 639-1 code.

          sample_rate: The desired output sample rate in Hz. Defaults to `24000` for all formats except
              `mulaw` which defaults to `8000`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        body = deepcopy_minimal(
            {
                "audio": audio,
                "voice": voice,
                "format": format,
                "language": language,
                "sample_rate": sample_rate,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["audio"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/v1/ai/speech/convert",
            body=maybe_transform(body, speech_convert_params.SpeechConvertParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def generate(
        self,
        *,
        text: str,
        voice: str,
        format: Literal["aac", "mp3", "raw", "ulaw", "wav", "webm"] | NotGiven = NOT_GIVEN,
        language: Literal[
            "auto",
            "ar",
            "de",
            "en",
            "es",
            "fr",
            "hi",
            "id",
            "it",
            "ja",
            "ko",
            "nl",
            "pl",
            "pt",
            "ru",
            "sv",
            "th",
            "tr",
            "uk",
            "ur",
            "vi",
            "zh",
        ]
        | NotGiven = NOT_GIVEN,
        model: Literal["blizzard"] | NotGiven = NOT_GIVEN,
        sample_rate: Literal[8000, 16000, 24000] | NotGiven = NOT_GIVEN,
        seed: int | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BinaryAPIResponse:
        """
        Generates speech from text and streams the audio as binary data chunks in
        real-time as they are generated.

        This is the recommended endpoint for most text-to-speech use cases. You can
        either stream the chunks for low-latency playback or collect all chunks to get
        the complete audio file.

        Args:
          text: The text to synthesize; max 5000 characters per request (including spaces).

          voice: The voice id of the voice to use; voice ids can be retrieved by calls to
              `List voices` or `Voice info`.

          format: The desired output format of the audio. If you are using a streaming endpoint,
              you'll generate audio faster by selecting a streamable format since chunks are
              encoded and returned as they're generated. For non-streamable formats, the
              entire audio will be synthesized before encoding.

              Streamable formats:

              - `mp3`: 96kbps MP3 audio.
              - `raw`: 32-bit floating point raw audio.
              - `ulaw`: 8-bit G711 µ-law audio with a WAV header.
              - `webm`: WebM format with Opus audio codec.

              Non-streamable formats:

              - `aac`: AAC audio codec.
              - `wav`: 16-bit PCM audio in WAV container.

          language: The desired language. Two letter ISO 639-1 code. Defaults to auto language
              detection.

          model: The model to use for synthesis. Learn more about models
              [here](https://docs.lmnt.com/guides/models).

          sample_rate: The desired output sample rate in Hz. Defaults to `24000` for all formats except
              `mulaw` which defaults to `8000`.

          seed: Seed used to specify a different take; defaults to random

          temperature: Influences how expressive and emotionally varied the speech becomes. Lower
              values (like 0.3) create more neutral, consistent speaking styles. Higher values
              (like 1.0) allow for more dynamic emotional range and speaking styles.

          top_p: Controls the stability of the generated speech. A lower value (like 0.3)
              produces more consistent, reliable speech. A higher value (like 0.9) gives more
              flexibility in how words are spoken, but might occasionally produce unusual
              intonations or speech patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._post(
            "/v1/ai/speech/bytes",
            body=maybe_transform(
                {
                    "text": text,
                    "voice": voice,
                    "format": format,
                    "language": language,
                    "model": model,
                    "sample_rate": sample_rate,
                    "seed": seed,
                    "temperature": temperature,
                    "top_p": top_p,
                },
                speech_generate_params.SpeechGenerateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def generate_detailed(
        self,
        *,
        text: str,
        voice: str,
        format: Literal["aac", "mp3", "raw", "ulaw", "wav", "webm"] | NotGiven = NOT_GIVEN,
        language: Literal[
            "auto",
            "ar",
            "de",
            "en",
            "es",
            "fr",
            "hi",
            "id",
            "it",
            "ja",
            "ko",
            "nl",
            "pl",
            "pt",
            "ru",
            "sv",
            "th",
            "tr",
            "uk",
            "ur",
            "vi",
            "zh",
        ]
        | NotGiven = NOT_GIVEN,
        model: Literal["blizzard"] | NotGiven = NOT_GIVEN,
        return_durations: bool | NotGiven = NOT_GIVEN,
        sample_rate: Literal[8000, 16000, 24000] | NotGiven = NOT_GIVEN,
        seed: int | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpeechGenerateDetailedResponse:
        """
        Generates speech from text and returns a JSON object that contains a
        **base64-encoded audio string** and optionally word-level durations
        (timestamps). This endpoint waits for the entire synthesis before responding, so
        it is not ideal for latency-sensitive applications.

        Args:
          text: The text to synthesize; max 5000 characters per request (including spaces).

          voice: The voice id of the voice to use; voice ids can be retrieved by calls to
              `List voices` or `Voice info`.

          format: The desired output format of the audio. If you are using a streaming endpoint,
              you'll generate audio faster by selecting a streamable format since chunks are
              encoded and returned as they're generated. For non-streamable formats, the
              entire audio will be synthesized before encoding.

              Streamable formats:

              - `mp3`: 96kbps MP3 audio.
              - `raw`: 32-bit floating point raw audio.
              - `ulaw`: 8-bit G711 µ-law audio with a WAV header.
              - `webm`: WebM format with Opus audio codec.

              Non-streamable formats:

              - `aac`: AAC audio codec.
              - `wav`: 16-bit PCM audio in WAV container.

          language: The desired language. Two letter ISO 639-1 code. Defaults to auto language
              detection.

          model: The model to use for synthesis. Learn more about models
              [here](https://docs.lmnt.com/guides/models).

          return_durations: If set as `true`, response will contain a durations object.

          sample_rate: The desired output sample rate in Hz. Defaults to `24000` for all formats except
              `mulaw` which defaults to `8000`.

          seed: Seed used to specify a different take; defaults to random

          temperature: Influences how expressive and emotionally varied the speech becomes. Lower
              values (like 0.3) create more neutral, consistent speaking styles. Higher values
              (like 1.0) allow for more dynamic emotional range and speaking styles.

          top_p: Controls the stability of the generated speech. A lower value (like 0.3)
              produces more consistent, reliable speech. A higher value (like 0.9) gives more
              flexibility in how words are spoken, but might occasionally produce unusual
              intonations or speech patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/ai/speech",
            body=maybe_transform(
                {
                    "text": text,
                    "voice": voice,
                    "format": format,
                    "language": language,
                    "model": model,
                    "return_durations": return_durations,
                    "sample_rate": sample_rate,
                    "seed": seed,
                    "temperature": temperature,
                    "top_p": top_p,
                },
                speech_generate_detailed_params.SpeechGenerateDetailedParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechGenerateDetailedResponse,
        )


class AsyncSpeechResource(AsyncAPIResource):
    @cached_property
    def sessions(self) -> AsyncSessionsResource:
        return AsyncSessionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lmnt-com/lmnt-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lmnt-com/lmnt-python#with_streaming_response
        """
        return AsyncSpeechResourceWithStreamingResponse(self)

    async def convert(
        self,
        *,
        audio: FileTypes,
        voice: str,
        format: Literal["aac", "mp3", "raw", "ulaw", "wav", "webm"] | NotGiven = NOT_GIVEN,
        language: Literal[
            "auto",
            "ar",
            "de",
            "en",
            "es",
            "fr",
            "hi",
            "id",
            "it",
            "ja",
            "ko",
            "nl",
            "pl",
            "pt",
            "ru",
            "sv",
            "th",
            "tr",
            "uk",
            "ur",
            "vi",
            "zh",
        ]
        | NotGiven = NOT_GIVEN,
        sample_rate: Literal[8000, 16000, 24000] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncBinaryAPIResponse:
        """
        Converts speech from one voice to another.

        Args:
          audio: The audio file to be converted into a new voice. Specify source language using
              the `language` parameter. Acceptable formats: `wav`, `mp3`. Max file size: 1 MB.

          voice: The voice id to convert the speech into. Voice ids can be retrieved by calls to
              `List voices` or `Voice info`.

          format: The desired output format of the audio. If you are using a streaming endpoint,
              you'll generate audio faster by selecting a streamable format since chunks are
              encoded and returned as they're generated. For non-streamable formats, the
              entire audio will be synthesized before encoding.

              Streamable formats:

              - `mp3`: 96kbps MP3 audio.
              - `raw`: 32-bit floating point raw audio.
              - `ulaw`: 8-bit G711 µ-law audio with a WAV header.
              - `webm`: WebM format with Opus audio codec.

              Non-streamable formats:

              - `aac`: AAC audio codec.
              - `wav`: 16-bit PCM audio in WAV container.

          language: The language of the source audio. Two letter ISO 639-1 code.

          sample_rate: The desired output sample rate in Hz. Defaults to `24000` for all formats except
              `mulaw` which defaults to `8000`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        body = deepcopy_minimal(
            {
                "audio": audio,
                "voice": voice,
                "format": format,
                "language": language,
                "sample_rate": sample_rate,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["audio"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/v1/ai/speech/convert",
            body=await async_maybe_transform(body, speech_convert_params.SpeechConvertParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def generate(
        self,
        *,
        text: str,
        voice: str,
        format: Literal["aac", "mp3", "raw", "ulaw", "wav", "webm"] | NotGiven = NOT_GIVEN,
        language: Literal[
            "auto",
            "ar",
            "de",
            "en",
            "es",
            "fr",
            "hi",
            "id",
            "it",
            "ja",
            "ko",
            "nl",
            "pl",
            "pt",
            "ru",
            "sv",
            "th",
            "tr",
            "uk",
            "ur",
            "vi",
            "zh",
        ]
        | NotGiven = NOT_GIVEN,
        model: Literal["blizzard"] | NotGiven = NOT_GIVEN,
        sample_rate: Literal[8000, 16000, 24000] | NotGiven = NOT_GIVEN,
        seed: int | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncBinaryAPIResponse:
        """
        Generates speech from text and streams the audio as binary data chunks in
        real-time as they are generated.

        This is the recommended endpoint for most text-to-speech use cases. You can
        either stream the chunks for low-latency playback or collect all chunks to get
        the complete audio file.

        Args:
          text: The text to synthesize; max 5000 characters per request (including spaces).

          voice: The voice id of the voice to use; voice ids can be retrieved by calls to
              `List voices` or `Voice info`.

          format: The desired output format of the audio. If you are using a streaming endpoint,
              you'll generate audio faster by selecting a streamable format since chunks are
              encoded and returned as they're generated. For non-streamable formats, the
              entire audio will be synthesized before encoding.

              Streamable formats:

              - `mp3`: 96kbps MP3 audio.
              - `raw`: 32-bit floating point raw audio.
              - `ulaw`: 8-bit G711 µ-law audio with a WAV header.
              - `webm`: WebM format with Opus audio codec.

              Non-streamable formats:

              - `aac`: AAC audio codec.
              - `wav`: 16-bit PCM audio in WAV container.

          language: The desired language. Two letter ISO 639-1 code. Defaults to auto language
              detection.

          model: The model to use for synthesis. Learn more about models
              [here](https://docs.lmnt.com/guides/models).

          sample_rate: The desired output sample rate in Hz. Defaults to `24000` for all formats except
              `mulaw` which defaults to `8000`.

          seed: Seed used to specify a different take; defaults to random

          temperature: Influences how expressive and emotionally varied the speech becomes. Lower
              values (like 0.3) create more neutral, consistent speaking styles. Higher values
              (like 1.0) allow for more dynamic emotional range and speaking styles.

          top_p: Controls the stability of the generated speech. A lower value (like 0.3)
              produces more consistent, reliable speech. A higher value (like 0.9) gives more
              flexibility in how words are spoken, but might occasionally produce unusual
              intonations or speech patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._post(
            "/v1/ai/speech/bytes",
            body=await async_maybe_transform(
                {
                    "text": text,
                    "voice": voice,
                    "format": format,
                    "language": language,
                    "model": model,
                    "sample_rate": sample_rate,
                    "seed": seed,
                    "temperature": temperature,
                    "top_p": top_p,
                },
                speech_generate_params.SpeechGenerateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def generate_detailed(
        self,
        *,
        text: str,
        voice: str,
        format: Literal["aac", "mp3", "raw", "ulaw", "wav", "webm"] | NotGiven = NOT_GIVEN,
        language: Literal[
            "auto",
            "ar",
            "de",
            "en",
            "es",
            "fr",
            "hi",
            "id",
            "it",
            "ja",
            "ko",
            "nl",
            "pl",
            "pt",
            "ru",
            "sv",
            "th",
            "tr",
            "uk",
            "ur",
            "vi",
            "zh",
        ]
        | NotGiven = NOT_GIVEN,
        model: Literal["blizzard"] | NotGiven = NOT_GIVEN,
        return_durations: bool | NotGiven = NOT_GIVEN,
        sample_rate: Literal[8000, 16000, 24000] | NotGiven = NOT_GIVEN,
        seed: int | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        top_p: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpeechGenerateDetailedResponse:
        """
        Generates speech from text and returns a JSON object that contains a
        **base64-encoded audio string** and optionally word-level durations
        (timestamps). This endpoint waits for the entire synthesis before responding, so
        it is not ideal for latency-sensitive applications.

        Args:
          text: The text to synthesize; max 5000 characters per request (including spaces).

          voice: The voice id of the voice to use; voice ids can be retrieved by calls to
              `List voices` or `Voice info`.

          format: The desired output format of the audio. If you are using a streaming endpoint,
              you'll generate audio faster by selecting a streamable format since chunks are
              encoded and returned as they're generated. For non-streamable formats, the
              entire audio will be synthesized before encoding.

              Streamable formats:

              - `mp3`: 96kbps MP3 audio.
              - `raw`: 32-bit floating point raw audio.
              - `ulaw`: 8-bit G711 µ-law audio with a WAV header.
              - `webm`: WebM format with Opus audio codec.

              Non-streamable formats:

              - `aac`: AAC audio codec.
              - `wav`: 16-bit PCM audio in WAV container.

          language: The desired language. Two letter ISO 639-1 code. Defaults to auto language
              detection.

          model: The model to use for synthesis. Learn more about models
              [here](https://docs.lmnt.com/guides/models).

          return_durations: If set as `true`, response will contain a durations object.

          sample_rate: The desired output sample rate in Hz. Defaults to `24000` for all formats except
              `mulaw` which defaults to `8000`.

          seed: Seed used to specify a different take; defaults to random

          temperature: Influences how expressive and emotionally varied the speech becomes. Lower
              values (like 0.3) create more neutral, consistent speaking styles. Higher values
              (like 1.0) allow for more dynamic emotional range and speaking styles.

          top_p: Controls the stability of the generated speech. A lower value (like 0.3)
              produces more consistent, reliable speech. A higher value (like 0.9) gives more
              flexibility in how words are spoken, but might occasionally produce unusual
              intonations or speech patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/ai/speech",
            body=await async_maybe_transform(
                {
                    "text": text,
                    "voice": voice,
                    "format": format,
                    "language": language,
                    "model": model,
                    "return_durations": return_durations,
                    "sample_rate": sample_rate,
                    "seed": seed,
                    "temperature": temperature,
                    "top_p": top_p,
                },
                speech_generate_detailed_params.SpeechGenerateDetailedParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechGenerateDetailedResponse,
        )


class SpeechResourceWithRawResponse:
    def __init__(self, speech: SpeechResource) -> None:
        self._speech = speech

        self.convert = to_custom_raw_response_wrapper(
            speech.convert,
            BinaryAPIResponse,
        )
        self.generate = to_custom_raw_response_wrapper(
            speech.generate,
            BinaryAPIResponse,
        )
        self.generate_detailed = to_raw_response_wrapper(
            speech.generate_detailed,
        )


class AsyncSpeechResourceWithRawResponse:
    def __init__(self, speech: AsyncSpeechResource) -> None:
        self._speech = speech

        self.convert = async_to_custom_raw_response_wrapper(
            speech.convert,
            AsyncBinaryAPIResponse,
        )
        self.generate = async_to_custom_raw_response_wrapper(
            speech.generate,
            AsyncBinaryAPIResponse,
        )
        self.generate_detailed = async_to_raw_response_wrapper(
            speech.generate_detailed,
        )


class SpeechResourceWithStreamingResponse:
    def __init__(self, speech: SpeechResource) -> None:
        self._speech = speech

        self.convert = to_custom_streamed_response_wrapper(
            speech.convert,
            StreamedBinaryAPIResponse,
        )
        self.generate = to_custom_streamed_response_wrapper(
            speech.generate,
            StreamedBinaryAPIResponse,
        )
        self.generate_detailed = to_streamed_response_wrapper(
            speech.generate_detailed,
        )


class AsyncSpeechResourceWithStreamingResponse:
    def __init__(self, speech: AsyncSpeechResource) -> None:
        self._speech = speech

        self.convert = async_to_custom_streamed_response_wrapper(
            speech.convert,
            AsyncStreamedBinaryAPIResponse,
        )
        self.generate = async_to_custom_streamed_response_wrapper(
            speech.generate,
            AsyncStreamedBinaryAPIResponse,
        )
        self.generate_detailed = async_to_streamed_response_wrapper(
            speech.generate_detailed,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import typing as _t

from . import types
from ._types import NOT_GIVEN, Omit, NoneType, NotGiven, Transport, ProxiesTypes
from ._utils import file_from_path
from ._client import Lmnt, Client, Stream, Timeout, AsyncLmnt, Transport, AsyncClient, AsyncStream, RequestOptions
from ._models import BaseModel
from ._version import __title__, __version__
from ._response import APIResponse as APIResponse, AsyncAPIResponse as AsyncAPIResponse
from ._constants import DEFAULT_TIMEOUT, DEFAULT_MAX_RETRIES, DEFAULT_CONNECTION_LIMITS
from ._exceptions import (
    APIError,
    LmntError,
    ConflictError,
    NotFoundError,
    APIStatusError,
    RateLimitError,
    APITimeoutError,
    BadRequestError,
    APIConnectionError,
    AuthenticationError,
    InternalServerError,
    PermissionDeniedError,
    UnprocessableEntityError,
    APIResponseValidationError,
)
from ._base_client import DefaultHttpxClient, DefaultAioHttpClient, DefaultAsyncHttpxClient
from ._utils._logs import setup_logging as _setup_logging

__all__ = [
    "types",
    "__version__",
    "__title__",
    "NoneType",
    "Transport",
    "ProxiesTypes",
    "NotGiven",
    "NOT_GIVEN",
    "Omit",
    "LmntError",
    "APIError",
    "APIStatusError",
    "APITimeoutError",
    "APIConnectionError",
    "APIResponseValidationError",
    "BadRequestError",
    "AuthenticationError",
    "PermissionDeniedError",
    "NotFoundError",
    "ConflictError",
    "UnprocessableEntityError",
    "RateLimitError",
    "InternalServerError",
    "Timeout",
    "RequestOptions",
    "Client",
    "AsyncClient",
    "Stream",
    "AsyncStream",
    "Lmnt",
    "AsyncLmnt",
    "file_from_path",
    "BaseModel",
    "DEFAULT_TIMEOUT",
    "DEFAULT_MAX_RETRIES",
    "DEFAULT_CONNECTION_LIMITS",
    "DefaultHttpxClient",
    "DefaultAsyncHttpxClient",
    "DefaultAioHttpClient",
]

if not _t.TYPE_CHECKING:
    from ._utils._resources_proxy import resources as resources

_setup_logging()

# Update the __module__ attribute for exported symbols so that
# error messages point to this module instead of the module
# it was originally defined in, e.g.
# lmnt._exceptions.NotFoundError -> lmnt.NotFoundError
__locals = locals()
for __name in __all__:
    if not __name.startswith("__"):
        try:
            __locals[__name].__module__ = "lmnt"
        except (TypeError, AttributeError):
            # Some of our exported symbols are builtins which we can't set attributes for.
            pass



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["VoiceCreateParams"]


class VoiceCreateParams(TypedDict, total=False):
    enhance: Required[bool]
    """
    For unclean audio with background noise, applies processing to attempt to
    improve quality. Default is `false` as this can also degrade quality in some
    circumstances.
    """

    files: Required[List[FileTypes]]
    """
    One or more input audio files to train the voice in the form of binary `wav`,
    `mp3`, `mp4`, `m4a`, or `webm` attachments.

    - Max attached files: 20.
    - Max total file size: 250 MB.
    """

    name: Required[str]
    """The display name for this voice"""

    description: str
    """A text description of this voice."""

    gender: str
    """A tag describing the gender of this voice. Has no effect on voice creation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Mapping, cast

import httpx

from ..types import voice_list_params, voice_create_params, voice_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.voice import Voice
from .._base_client import make_request_options
from ..types.voice_list_response import VoiceListResponse
from ..types.voice_delete_response import VoiceDeleteResponse
from ..types.voice_update_response import VoiceUpdateResponse

__all__ = ["VoicesResource", "AsyncVoicesResource"]


class VoicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lmnt-com/lmnt-python#accessing-raw-response-data-eg-headers
        """
        return VoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lmnt-com/lmnt-python#with_streaming_response
        """
        return VoicesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        enhance: bool,
        files: List[FileTypes],
        name: str,
        description: str | NotGiven = NOT_GIVEN,
        gender: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Voice:
        """
        Submits a request to create a voice with a supplied voice configuration and a
        batch of input audio data.

        Args:
          enhance: For unclean audio with background noise, applies processing to attempt to
              improve quality. Default is `false` as this can also degrade quality in some
              circumstances.

          files: One or more input audio files to train the voice in the form of binary `wav`,
              `mp3`, `mp4`, `m4a`, or `webm` attachments.

              - Max attached files: 20.
              - Max total file size: 250 MB.

          name: The display name for this voice

          description: A text description of this voice.

          gender: A tag describing the gender of this voice. Has no effect on voice creation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "enhance": enhance,
                "files": files,
                "name": name,
                "description": description,
                "gender": gender,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # Remove the files from the body since they are now in extracted_files
        body.pop("files", None)
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/v1/ai/voice",
            body=maybe_transform(body, voice_create_params.VoiceCreateParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Voice,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Voice:
        """
        Returns details of a specific voice.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/v1/ai/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Voice,
        )

    def update(
        self,
        id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        gender: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        starred: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> VoiceUpdateResponse:
        """Updates metadata for a specific voice.

        Only provided fields will be changed.

        Args:
          description: A description of this voice.

          gender: A tag describing the gender of this voice, e.g. `male`, `female`, `nonbinary`.

          name: The display name for this voice.

          starred: If `true`, adds this voice to your starred list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/v1/ai/voice/{id}",
            body=maybe_transform(
                {
                    "description": description,
                    "gender": gender,
                    "name": name,
                    "starred": starred,
                },
                voice_update_params.VoiceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceUpdateResponse,
        )

    def list(
        self,
        *,
        owner: str | NotGiven = NOT_GIVEN,
        starred: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> VoiceListResponse:
        """
        Returns a list of voices available to you.

        Args:
          owner: Which owner's voices to return. Choose from `system`, `me`, or `all`.

          starred: If true, only returns voices that you have starred.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/ai/voice/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "owner": owner,
                        "starred": starred,
                    },
                    voice_list_params.VoiceListParams,
                ),
            ),
            cast_to=VoiceListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> VoiceDeleteResponse:
        """Deletes a voice and cancels any pending operations on it.

        Cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/v1/ai/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDeleteResponse,
        )


class AsyncVoicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lmnt-com/lmnt-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lmnt-com/lmnt-python#with_streaming_response
        """
        return AsyncVoicesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        enhance: bool,
        files: List[FileTypes],
        name: str,
        description: str | NotGiven = NOT_GIVEN,
        gender: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Voice:
        """
        Submits a request to create a voice with a supplied voice configuration and a
        batch of input audio data.

        Args:
          enhance: For unclean audio with background noise, applies processing to attempt to
              improve quality. Default is `false` as this can also degrade quality in some
              circumstances.

          files: One or more input audio files to train the voice in the form of binary `wav`,
              `mp3`, `mp4`, `m4a`, or `webm` attachments.

              - Max attached files: 20.
              - Max total file size: 250 MB.

          name: The display name for this voice

          description: A text description of this voice.

          gender: A tag describing the gender of this voice. Has no effect on voice creation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "enhance": enhance,
                "files": files,
                "name": name,
                "description": description,
                "gender": gender,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # Remove the files from the body since they are now in extracted_files
        body.pop("files", None)
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/v1/ai/voice",
            body=await async_maybe_transform(body, voice_create_params.VoiceCreateParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Voice,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Voice:
        """
        Returns details of a specific voice.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/v1/ai/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Voice,
        )

    async def update(
        self,
        id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        gender: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        starred: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> VoiceUpdateResponse:
        """Updates metadata for a specific voice.

        Only provided fields will be changed.

        Args:
          description: A description of this voice.

          gender: A tag describing the gender of this voice, e.g. `male`, `female`, `nonbinary`.

          name: The display name for this voice.

          starred: If `true`, adds this voice to your starred list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/v1/ai/voice/{id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "gender": gender,
                    "name": name,
                    "starred": starred,
                },
                voice_update_params.VoiceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceUpdateResponse,
        )

    async def list(
        self,
        *,
        owner: str | NotGiven = NOT_GIVEN,
        starred: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> VoiceListResponse:
        """
        Returns a list of voices available to you.

        Args:
          owner: Which owner's voices to return. Choose from `system`, `me`, or `all`.

          starred: If true, only returns voices that you have starred.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/ai/voice/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "owner": owner,
                        "starred": starred,
                    },
                    voice_list_params.VoiceListParams,
                ),
            ),
            cast_to=VoiceListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> VoiceDeleteResponse:
        """Deletes a voice and cancels any pending operations on it.

        Cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/v1/ai/voice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceDeleteResponse,
        )


class VoicesResourceWithRawResponse:
    def __init__(self, voices: VoicesResource) -> None:
        self._voices = voices

        self.create = to_raw_response_wrapper(
            voices.create,
        )
        self.retrieve = to_raw_response_wrapper(
            voices.retrieve,
        )
        self.update = to_raw_response_wrapper(
            voices.update,
        )
        self.list = to_raw_response_wrapper(
            voices.list,
        )
        self.delete = to_raw_response_wrapper(
            voices.delete,
        )


class AsyncVoicesResourceWithRawResponse:
    def __init__(self, voices: AsyncVoicesResource) -> None:
        self._voices = voices

        self.create = async_to_raw_response_wrapper(
            voices.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            voices.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            voices.update,
        )
        self.list = async_to_raw_response_wrapper(
            voices.list,
        )
        self.delete = async_to_raw_response_wrapper(
            voices.delete,
        )


class VoicesResourceWithStreamingResponse:
    def __init__(self, voices: VoicesResource) -> None:
        self._voices = voices

        self.create = to_streamed_response_wrapper(
            voices.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            voices.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            voices.update,
        )
        self.list = to_streamed_response_wrapper(
            voices.list,
        )
        self.delete = to_streamed_response_wrapper(
            voices.delete,
        )


class AsyncVoicesResourceWithStreamingResponse:
    def __init__(self, voices: AsyncVoicesResource) -> None:
        self._voices = voices

        self.create = async_to_streamed_response_wrapper(
            voices.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            voices.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            voices.update,
        )
        self.list = async_to_streamed_response_wrapper(
            voices.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            voices.delete,
        )



from pathlib import Path

from lmnt import Lmnt

client = Lmnt()

response = client.voices.create(
    name="My Voice",
    files=[Path("sample1.wav"), Path("sample2.wav")],
    enhance=False,
)

print(response)
